'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;

  return {
    visitor: {
      CallExpression: function CallExpression(path, node) {
        if (isValidRequireStatement(path, t)) {
          var rawPath = path.node.arguments[0].value;
          if (isImportingAnElmAsset(rawPath)) {
            var newRequiredPath = getNewPath(rawPath, node);
            transform(path, newRequiredPath);
          }
        }
      }
    }
  };
};

var _babelTypes = require('babel-types');

var _nodeElmCompiler = require('node-elm-compiler');

var _nodeElmCompiler2 = _interopRequireDefault(_nodeElmCompiler);

var _path = require('path');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var temp = require('temp').track();

var isValidRequireStatement = function isValidRequireStatement(path, types) {
  var _path$node = path.node,
      calleeName = _path$node.callee.name,
      args = _path$node.arguments;


  var isRequireStatement = calleeName === 'require';
  var hasStingLiteralAsFirstArg = args.length && types.isStringLiteral(args[0]);

  return isRequireStatement && hasStingLiteralAsFirstArg;
};

var getNewPath = function getNewPath(rawPath, node) {
  var assetAbsolutePath = getAssetAbsolutePath(rawPath, node);
  return compile(assetAbsolutePath);
};

var transform = function transform(path, newRequiredPath) {
  path.node.arguments = [(0, _babelTypes.StringLiteral)(newRequiredPath)];
};

var isImportingAnElmAsset = function isImportingAnElmAsset(rawPath) {
  return (/.elm$/.test(rawPath)
  );
};

var getAssetAbsolutePath = function getAssetAbsolutePath(rawPath, node) {
  var importingFilePath = node.file.opts.filenameRelative;
  var containingDir = (0, _path.dirname)(importingFilePath);

  return (0, _path.resolve)(containingDir, rawPath);
};

var compile = function compile(fileToCompileAbsolutePath) {
  var tempFilePath = temp.openSync({ suffix: '.js' }).path;
  var options = { output: tempFilePath };

  _nodeElmCompiler2.default.compileSync(fileToCompileAbsolutePath, options);

  return tempFilePath;
};